import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'single_key_pressed_intent.dart';
import 'single_key_shortcut_descriptor.dart';

class SingleKeyShortcut extends StatelessWidget {
  const SingleKeyShortcut({
    Key? key,
    required this.child,
    required this.shortcutDescriptors,
  }) : super(key: key);

  final Widget child;
  final List<SingleKeyShortcutDescriptor> shortcutDescriptors;

  @override
  Widget build(BuildContext context) {
    Map<ShortcutActivator, Intent> shortcuts = {};
    Map<Type, Action<Intent>> actions = {};

    for (var shortcutDescriptor in shortcutDescriptors) {
      if (shortcutDescriptor.keySet == null) {
        final key = [
          shortcutDescriptor.alt ? LogicalKeyboardKey.alt : null,
          shortcutDescriptor.control ? LogicalKeyboardKey.control : null,
          shortcutDescriptor.meta ? LogicalKeyboardKey.meta : null,
          shortcutDescriptor.shift ? LogicalKeyboardKey.shift : null,
        ];
        final Set<LogicalKeyboardKey> set = key.where((element) => element != null).map((e) => e!).toSet();
        final keySet = LogicalKeySet.fromSet(set);

        final intent = shortcutDescriptor.getintent(LogicalKeyboardKey.abort);
        final type = intent.runtimeType;
        final Map<Type, Action<Intent>> action = {type: shortcutDescriptor.action};
        actions.addAll(action);
        final shortcut = {
          keySet: intent, // SingleKeyPressedIntent(e),
        };
        shortcuts.addAll(shortcut);
      } else {
        final map = List<Map<ShortcutActivator, SingleKeyPressedIntent>>.from(shortcutDescriptor.keySet!.keys.map(
          (e) {
            final intent = shortcutDescriptor.getintent(e);
            final type = intent.runtimeType;
            final Map<Type, Action<Intent>> action = {type: shortcutDescriptor.action};
            actions.addAll(action);
            return {
              SingleActivator(
                e,
                control: shortcutDescriptor.control,
                alt: shortcutDescriptor.alt,
                meta: shortcutDescriptor.meta,
                shift: shortcutDescriptor.shift,
              ): intent, // SingleKeyPressedIntent(e),
            };
          },
        )
            // .cast<Map<ShortcutActivator, SingleKeyPressedIntent>>(),
            );
        final what = Map.fromEntries(map.map((e) => e.entries).reduce((value, element) => [...value, ...element]));
        shortcuts.addAll(what);
      }
    }

    // final what = Map.fromEntries(map.map((e) => e.entries).reduce((value, element) => [...value, ...element]));

    return Shortcuts(
      // manager: LoggingShortcutManager(),
      shortcuts: shortcuts, // what,
      child: Actions(
        // dispatcher: LoggingActionDispatcher(),
        actions: actions, // {SingleKeyPressedIntent: shortcutDescriptor.action},
        // actions: <Type, Action<Intent>>{
        //   SingleKeyPressedIntent: ScreenshotAction(),
        // },
        child: Focus(
          autofocus: true,
          child: child,
        ),
      ),
    );
  }
}

// class SingleKeyShortcut extends StatelessWidget {
//   SingleKeyShortcut({
//     Key? key,
//     LogicalKeySet? keySet,
//     LogicalKeyboardKey? logicalKeyboardKey,
//     required this.child,
//     this.control = false,
//     this.shift = false,
//     this.alt = false,
//     this.meta = false,
//   })  : assert(logicalKeyboardKey != null || keySet != null),
//         keySet = keySet ?? LogicalKeySet(logicalKeyboardKey!),
//         super(key: key);

//   final Widget child;
//   final LogicalKeySet keySet;
//   final bool control;
//   final bool shift;
//   final bool alt;
//   final bool meta;

//   @override
//   Widget build(BuildContext context) {
//     List<Map<ShortcutActivator, Intent>> map = List<Map<ShortcutActivator, Intent>>.from(
//       keySet.keys.map(
//         (e) => {
//           SingleActivator(
//             e,
//             control: control,
//             alt: alt,
//             meta: meta,
//             shift: shift,
//           ): SingleKeyPressedIntent(e),
//         },
//       ),
//     );
//     final what = Map.fromEntries(map.map((e) => e.entries).reduce((value, element) => [...value, ...element]));

//     return Shortcuts(
//       manager: LoggingShortcutManager(),
//       shortcuts: what,
//       child: Actions(
//         dispatcher: LoggingActionDispatcher(),
//         actions: <Type, Action<Intent>>{
//           SingleKeyPressedIntent: ScreenshotAction(),
//         },
//         child: Focus(
//           autofocus: true,
//           child: child,
//         ),
//       ),
//     );
//   }
// }
